@@ -161,6 +161,8 @@ void * download_thread (void *arg)
   if (sidtune_is_ok (args->file_path))
   {
     add_song_to_playlist (args->file_path);
+
+    set_status_text ("SONG WAS DOWNLOADED", NULL);
   }
   else
   {
@@ -253,7 +255,7 @@ download_and_remove (GtkTreeView       *tree_view,
 
       snprintf (file_path, 256, "%s/%s", *p, name);
 
-      printf("msid downloading [%s]\n", file_path);
+      // printf("msid downloading [%s]\n", file_path);
 
       /* --- LAUNCH THREAD HERE --- */
 
@@ -353,6 +355,7 @@ make_search (GtkWidget *song_list, const char *needle)
   GtkTreeModel *model=NULL;
   GtkTreeIter iter;
   GError *error=NULL;
+  int k;
 
   // plugin _needs_ to exist
   extern msid_search_plugin *msid_active_search_plugin;
@@ -369,6 +372,14 @@ make_search (GtkWidget *song_list, const char *needle)
   search_args.list = song_list;
   search_args.needle = g_strdup(needle);
 
+  // replace spaces with '_'
+  // (usual format in sid files)
+  for (k=0; k<strlen(search_args.needle); k++) {
+    if (search_args.needle[k] == ' ') {
+      search_args.needle[k] = '_';
+    }
+  }
+
   G_LOCK (search_active);
   search_active = TRUE;
   G_UNLOCK (search_active);
@@ -514,12 +525,13 @@ create_loader_page (msid_search_plugin *plugin,
    * create widgets
    */
 
-  // TODO - fix bugs with pannable area
-  //scroll_win = hildon_pannable_area_new();
-
+#ifdef MAEMO5
+  scroll_win = hildon_pannable_area_new();
+#else
   scroll_win = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll_win),
   				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+#endif
 
   search_button = create_ui_button ("search");
   search_entry  = gtk_entry_new();
