@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Tapani Pälli <lemody@c64.org>
+ * Copyright (C) 2008 Tapani Pälli <lemody@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,9 +44,7 @@
 #include <sidplay/fformat.h>
 #include <sidplay/myendian.h>
 
-#include <vector>
-
-#define PROGRAM_VERSION "0.3.9"
+#define PROGRAM_VERSION "0.4"
 #define PROGRAM_NAME "msid - " PROGRAM_VERSION
 
 #define APP_WINDOW_HEIGHT 400 // ~hildonwindow workspace
@@ -66,6 +64,8 @@ extern GtkWidget *player_read_button;
 extern GtkWidget *player_del_button;
 extern GtkWidget *subsong_selector;
 
+static GtkWidget *config_dialog = NULL;
+
 extern GtkWidget *config_audio_plugin_button;
 extern GtkWidget *config_search_plugin_button;
 extern GtkWidget *config_audio_plugin_combo;
@@ -78,23 +78,20 @@ extern GtkWidget *hvsc_path_label;
 extern GtkWidget *image_button;
 extern GtkWidget *anim_control_button;
 
-// DANGER - this _has_ to point to the current plugin
-msid_search_plugin *msid_active_search_plugin;
-
-static GtkWidget *config_dialog = NULL;
+msid_search_plugin *msid_active_search_plugin = NULL;
 
 static gboolean ignore_subsong_change = FALSE;
 
-static int image_page_num=0;
+static int image_page_num = 0;
 
-static int SUBSONG = 1;
+static int SUBSONG = 0;
 
 char download_path[256];
 
 static GtkTreePath *current_selection = NULL;
 
 
-/* these settings are used by default *only* if everything else fails */
+// these settings are used by default *only* if everything else fails
 #define DEFAULT_AUDIO_PLUGIN "libmsid_pulse.so"
 #define DEFAULT_SEARCH_PLUGIN "libmsid_c64org.so"
 static const gchar default_config[] =
@@ -109,9 +106,8 @@ typedef struct msid_machine
   uword song_no;
 } msid_machine;
 
-/*
- * states for player thread
- */
+
+// states for player thread
 enum
   {
     PLAY = 0,
@@ -120,12 +116,15 @@ enum
     READY
   };
 
+// mutex for state communication between main and player thread
 G_LOCK_DEFINE_STATIC(msid_state);
 static volatile unsigned int msid_state = READY;
+
+// mutex for pausing the playback of player thread
 G_LOCK_DEFINE_STATIC(msid_pause);
 static volatile unsigned int msid_pause = 0;
 
-
+// interface for audio and search plugin management
 typedef struct _msid_plugin_iface
 {
   void *dl_handle;
@@ -134,23 +133,21 @@ typedef struct _msid_plugin_iface
   destroy_t* destroy_plugin;
 } msid_plugin_iface;
 
-// interface to c++ objects
+// interfaces to c++ objects
 msid_plugin_iface audio_iface;
 msid_plugin_iface search_iface;
 
+// function declarations
+static void init_msid_machine (msid_machine *m);
+static void select_song (GtkTreeView *treeview, gpointer data);
+static void modify_subsong_selector();
+void update_status_text (const char *text, const char *color);
 
-static void
-init_msid_machine (msid_machine *m);
-
-void
-update_status_text (const char *text, const char *color);
-
-static void
-select_song (GtkTreeView *treeview, gpointer data);
-
-static void
-modify_subsong_selector();
 
+/*
+ * playback thread, reads data from libsidplay and feeds
+ * it to audio plugin, listens to state changes made by main thread.
+ */
 void *player_thread(void *arg)
 {
   unsigned int ready = 0;
@@ -168,7 +165,7 @@ void *player_thread(void *arg)
 #endif
 #endif
   
-  // final sanity check
+  // final sanity check, this should never happen
   if (!m->song) {
     G_LOCK(msid_state);
     msid_state = READY;
@@ -177,7 +174,8 @@ void *player_thread(void *arg)
     return NULL;
   }
 
-  init_msid_machine (m); /* init driver */
+  // init audio driver
+  init_msid_machine (m);
 
   buffer = new ubyte[m->driver->bsize()];
 
@@ -223,9 +221,7 @@ void *player_thread(void *arg)
 
 
 
-/*
- * set fullscreen state for window
- */
+// set fullscreen state for window (N770, N800, N810)
 static gint
 msid_on_key_release_event (GtkWidget *widget, GdkEventKey *event)
 {
@@ -251,16 +247,18 @@ msid_on_key_release_event (GtkWidget *widget, GdkEventKey *event)
   return FALSE;
 }
 
+// default text to show as status after specified timeout
 static gboolean
 set_ready_text_timeout (gpointer data)
 {
   gdk_threads_enter();
-  update_status_text ("READY.", C64_FG);
+  update_status_text("READY.", C64_FG);
   gdk_threads_leave();
 
   return FALSE;
 }
 
+// change status text
 void
 update_status_text (const char *text, const char *c)
 {
@@ -280,9 +278,7 @@ update_status_text (const char *text, const char *c)
   gtk_label_set_markup (GTK_LABEL(status_text), buffer);
 }
 
-/*
- * santa's little helper
- */
+// pack to vbox with a separator
 static void
 append_to_vbox (GtkWidget *box, GtkWidget *w,
 		gboolean a, gboolean b)
@@ -336,17 +332,16 @@ init_msid_plugin (msid_plugin_iface *iface,
   return 1;
 }
 
+// close callbacks for plugins in case cleanup needed
 static void
 close_msid_plugin (msid_plugin_iface *iface)
 {
-  iface -> destroy_plugin (iface -> plugin_object);
-  dlclose (iface -> dl_handle);
+  iface->destroy_plugin(iface->plugin_object);
+  dlclose(iface->dl_handle);
 }
 
-/*
- * strange looking configuration function
- */
 
+// initialization of the audio driver and libsidplay
 static void
 init_msid_machine (msid_machine *m)
 {
@@ -397,7 +392,6 @@ load_screenshot (GtkWidget *widget,
 
   gtk_widget_set_sensitive (widget, FALSE);
   set_game_screenshot (filepath);
-
 }
 
 static void
@@ -508,7 +502,7 @@ load_search_plugin (GtkWidget *widget,
   }
 }
 
-
+// pause callback, sets mutex state according to pause button state
 static void
 pause_song (GtkWidget *widget,
 	    gpointer data)
@@ -611,44 +605,6 @@ play_stop_song (GtkWidget *widget,
   g_thread_create(player_thread, (void*) data, FALSE, &error);
 
   return;
-
-
-  /*
-
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(player_song_list));
-  if (gtk_tree_selection_get_selected(selection, &model, &iter))
-  {
-    path = gtk_tree_model_get_path(model, &iter);
-    if(!path)
-    {
-      fprintf(stderr, "no path\n");
-      return;
-    }
-
-    column = gtk_tree_view_get_column(GTK_TREE_VIEW(player_song_list), 0);
-    gtk_tree_selection_get_selected(selection, &model, &iter);
-    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
-    gtk_tree_path_free(path);
-
-    if(!filepath)
-    {
-      return;
-    }
-    else
-    {
-      m->song = filepath;
-      m->song_no = SUBSONG;
-
-      g_thread_create(player_thread, (void*) data, FALSE, &error);
-      return;
-    }
-  } else {
-    fprintf(stderr, "no selection\n");
-  }
-
-  */
-
-
 }
 
 
@@ -656,7 +612,6 @@ static void
 quit_msid (GtkMenuItem *menuitem, gpointer data)
 {
   stop_sid_machine();
-
   gtk_main_quit();
 }
 
@@ -727,7 +682,6 @@ play_selection (GtkTreeView       *tree_view,
 
       }
     }
-
     break;
   }
 }
@@ -741,7 +695,6 @@ handle_selection (GtkTreeView       *tree_view,
   GtkTreeModel     *model;
 
   model = gtk_tree_view_get_model(tree_view);
-
   play_selection(tree_view, path, column, user_data);
 }
 
@@ -766,6 +719,32 @@ get_subsong_amount()
   return amount;
 }
 
+
+static void replay(msid_machine *m)
+{
+  G_LOCK(msid_state);
+
+  if (msid_state != READY) {
+
+    G_UNLOCK(msid_state);
+
+    stop_sid_machine();
+
+    G_LOCK(msid_state);
+    if (msid_state != READY) {
+      fprintf(stderr, "stop did not work ok :/\n");
+    }
+    G_UNLOCK(msid_state);
+
+    m->song_no = SUBSONG;
+
+    play_stop_song(NULL, m);
+    return;
+  }
+  G_UNLOCK(msid_state);
+}
+
+
 static void
 subsong_changed (GtkComboBox *widget, gpointer data)
 {
@@ -788,27 +767,8 @@ subsong_changed (GtkComboBox *widget, gpointer data)
     return;
   }
 
-  G_LOCK(msid_state);
-  if (msid_state != READY) {
-
-    G_UNLOCK(msid_state);
-
-    stop_sid_machine();
-
-    G_LOCK(msid_state);
-    if (msid_state != READY) {
-      fprintf(stderr, "stop did not work ok :/\n");
-    }
-    G_UNLOCK(msid_state);
-
-    m->song_no = SUBSONG;
-
-    play_stop_song(NULL, data);
-
-    return;
-  }
-  G_UNLOCK(msid_state);
-
+  // replay sid with different subsong
+  replay(m);
 }
 
 
@@ -827,14 +787,16 @@ modify_subsong_selector()
     printf ("[%s] - no subsongs found\n", __func__);
     return;
   }
-  prev_amount = gtk_tree_model_iter_n_children (model, NULL);
+  // account for SONG
+  prev_amount = gtk_tree_model_iter_n_children (model, NULL) - 1;
+  //printf("%d %d\n", prev_amount, new_amount);
 
   if (prev_amount < new_amount) {
     // add new entries if needed
-    while (prev_amount < new_amount + 1) {
+    while (prev_amount < new_amount) {
+      prev_amount++;
       sprintf(str, "%d", prev_amount);
       gtk_combo_box_append_text(GTK_COMBO_BOX(subsong_selector), str);
-      prev_amount++;
     }
 
     // set same subsong as was selected before!
@@ -843,7 +805,7 @@ modify_subsong_selector()
 
   else if (prev_amount > new_amount) {
     // remove entries if needed
-    while (prev_amount >= new_amount) {
+    while (prev_amount > new_amount) {
       gtk_combo_box_remove_text(GTK_COMBO_BOX(subsong_selector),
 				prev_amount);
       prev_amount--;
@@ -854,7 +816,7 @@ modify_subsong_selector()
     // ignore this change as it's program making it
     ignore_subsong_change = TRUE;
 
-    if (SUBSONG < new_amount)
+    if (SUBSONG <= new_amount)
       gtk_combo_box_set_active (GTK_COMBO_BOX(subsong_selector), SUBSONG);
     else
       gtk_combo_box_set_active (GTK_COMBO_BOX(subsong_selector), 0);
@@ -862,6 +824,7 @@ modify_subsong_selector()
 
 }
 
+// delete current sid song with confirmation dialog
 static void
 delete_current_selection (void)
 {
@@ -942,7 +905,7 @@ delete_current_selection (void)
     }
 }
 
-
+// delete button callback
 static void
 delete_song (GtkWidget *widget,
 	     gpointer data)
@@ -950,10 +913,11 @@ delete_song (GtkWidget *widget,
   delete_current_selection();
 }
 
+// keyboard callback
 static gboolean
-key_on_selection (GtkWidget   *widget,
-		  GdkEventKey *event,
-		  gpointer     user_data)
+key_on_selection(GtkWidget   *widget,
+		 GdkEventKey *event,
+		 gpointer     user_data)
 {
   switch (event->keyval)
   {
@@ -984,6 +948,7 @@ select_song (GtkTreeView *treeview,
 }
 
 
+// notebook page was changed, stop screenshot animation
 static void
 page_changed (GtkNotebook     *notebook,
 	      GtkNotebookPage *new_page,
@@ -991,13 +956,7 @@ page_changed (GtkNotebook     *notebook,
 	      gpointer         user_data)
 {
   if (page_num != image_page_num) {
-
     stop_screenshot_animation();
-    // if (page_num == player_page_num) update_status_text ("playing ...");
-  } else {
-    // autoplay?
-    //
-    ; // play_screenshot_animation();    
   }
 }
 
@@ -1008,7 +967,6 @@ program_is_destroyed (gpointer data, GObject *program)
 }
 
 
-
 static void
 show_from_panel (GtkStatusIcon *status_icon,
 		 gpointer       user_data)
@@ -1074,11 +1032,11 @@ launch_browser (GtkLinkButton *button,
     gnome_vfs_mime_get_default_application_for_uri
     (link_, "text/html");
 
-  list = g_list_append (list, g_strdup (link_));
-  gnome_vfs_mime_application_launch (app, list);
+  list = g_list_append(list, g_strdup (link_));
+  gnome_vfs_mime_application_launch(app, list);
 
-  g_list_foreach (list, free_args, NULL);
-  g_list_free (list);
+  g_list_foreach(list, free_args, NULL);
+  g_list_free(list);
 
   free(app);
 }
@@ -1087,36 +1045,35 @@ launch_browser (GtkLinkButton *button,
 GtkWidget *
 append_sponsor_links (GtkWidget *dialog)
 {
-  GtkWidget *c64org = gtk_link_button_new_with_label ("http://www.c64.org", "c64.org");
-  GtkWidget *c64com = gtk_link_button_new_with_label ("http://www.c64.com", "c64.com");
+  GtkWidget *c64org = gtk_link_button_new_with_label("http://www.c64.org", "c64.org");
+  GtkWidget *c64com = gtk_link_button_new_with_label("http://www.c64.com", "c64.com");
 
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), GTK_WIDGET(c64org));
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), GTK_WIDGET(c64com));
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), GTK_WIDGET(c64org));
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), GTK_WIDGET(c64com));
 
   return dialog;
 }
 
 void
-show_info_dialog (char *title, char *txt1, char *txt2)
+show_info_dialog (const char *title, const char *txt1, const char *txt2)
 {
-  GtkWidget *d;
+  GtkWidget *dialog;
 
-  d = gtk_message_dialog_new_with_markup (GTK_WINDOW(app_window),
-                                          GTK_DIALOG_DESTROY_WITH_PARENT,
-                                          GTK_MESSAGE_INFO,
-                                          GTK_BUTTONS_CLOSE,
-                                          txt1);
-  gtk_window_set_title(GTK_WINDOW(d), title);
+  dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(app_window),
+					       GTK_DIALOG_DESTROY_WITH_PARENT,
+					       GTK_MESSAGE_INFO,
+					       GTK_BUTTONS_CLOSE,
+					       txt1);
+  gtk_window_set_title(GTK_WINDOW(dialog), title);
 
-  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(d),
-					   txt2);
+  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), txt2);
 
 #ifndef HILDON
-  gtk_window_set_position (GTK_WINDOW(d), GTK_WIN_POS_CENTER_ON_PARENT);
+  gtk_window_set_position (GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 #endif
 
-  gtk_dialog_run (GTK_DIALOG(d));
-  gtk_widget_destroy(d);
+  gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_widget_destroy(dialog);
 }
 
 void
@@ -1128,7 +1085,7 @@ show_about_dialog (GtkMenuItem *menuitem, gpointer data)
 
   char buffer[256];
 
-  char txt[] = "MSID uses libsidplay1, a SID chip emulator by Michael Schwendt. MSID UI was programmed by lemody@c64.org\n\nMany thanks and respect to the admins of c64.org and c64.com for letting MSID use their resources.";
+  char txt[] = "MSID uses libsidplay1, a SID chip emulator by Michael Schwendt. MSID UI was programmed by lemody@gmail.com\n\nMany thanks and respect to the admins of c64.org and c64.com for letting MSID use their resources.";
 
   GtkWidget *dialog = gtk_dialog_new_with_buttons ("About",
 						   GTK_WINDOW(app_window),
@@ -1208,20 +1165,18 @@ show_config_dialog (GtkMenuItem *menuitem, gpointer data)
 
   config_dialog = NULL;
 
-  /*
-   * unpack config page
-   */
+  /* unpack config page */
 
   gtk_container_remove (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), GTK_WIDGET(data));
   gtk_widget_destroy (dialog);
 }
 
+
 /*
  * very experimental code
- *
  */
 static void
-colorize_widget (char *widget)
+colorize_widget (const char *widget)
 {
   char rc_buffer[512];
 
@@ -1280,14 +1235,14 @@ class "GtkWindow" style "window"
 }
 
 static void output_mode_changed (GtkToggleButton *togglebutton,
-				 gpointer         user_data)
+				 gpointer user_data)
 {
   // HACK - restart of the engine needed currently
   // TODO - figure out nice way without complete restart
 
-  update_status_text ("OUTPUT MODE CHANGED", C64_FG);
+  update_status_text("OUTPUT MODE CHANGED", C64_FG);
 
-  subsong_changed (NULL, user_data);
+  replay((msid_machine*)user_data);
 }
 
 
@@ -1348,43 +1303,40 @@ int main (int argc, char **argv)
 
   msid_state = READY;
 
-  //  memset(&machine, 0, sizeof(msid_machine));
-
   machine.song = 0;
 
   g_thread_init(NULL);
   gdk_threads_init();
   gdk_threads_enter();
 
-  gtk_init (&argc, &argv);
+  gtk_init(&argc, &argv);
 
-  gnome_vfs_init ();
+  gnome_vfs_init();
 
   gtk_link_button_set_uri_hook (launch_browser, NULL, NULL);
 
   /* hildon theming overrides colorization ... buhuu. */
-  //#ifndef HILDON
-  //colorize_widget ("GtkTreeView");
-  //colorize_widget ("GtkButton");
-  //#endif
+#ifndef HILDON
+  colorize_widget("GtkTreeView");
+  colorize_widget("GtkButton");
+#endif
 
   config_file = g_key_file_new();
 
   /* TODO - try configuration file first */
 
   error=NULL;
-  g_key_file_load_from_file (config_file,
-			     CONFIG_FILE_FULL_PATH,
-			     G_KEY_FILE_NONE,
-			     &error);
+  g_key_file_load_from_file(config_file,
+			    CONFIG_FILE_FULL_PATH,
+			    G_KEY_FILE_NONE,
+			    &error);
   /* load default configuration */
-  if (error)
-  {
+  if (error) {
     error=NULL;
-    g_key_file_load_from_data (config_file,
-			       default_config,
-			       sizeof(default_config),
-			       G_KEY_FILE_NONE, &error);
+    g_key_file_load_from_data(config_file,
+			      default_config,
+			      sizeof(default_config),
+			      G_KEY_FILE_NONE, &error);
   }
 
   // AUDIO PLUGIN
@@ -1392,23 +1344,17 @@ int main (int argc, char **argv)
   config_value = g_key_file_get_string
     (config_file, "msid_config", "audio_plugin", &error);
 
-  if (config_value)
-  {
+  if (config_value) {
     snprintf (buffer, 256, "%s/audio/%s", PLUGIN_FILE_PATH, config_value);
     g_free (config_value);
 
-    if (g_file_test (buffer, G_FILE_TEST_EXISTS))
-    {
+    if (g_file_test (buffer, G_FILE_TEST_EXISTS)) {
       init_msid_plugin (&audio_iface, buffer);
-    }
-    else
-    {
+    } else {
       fprintf(stderr ,"audio plugin init failed!\n");
       //      goto away;
     }
-  }
-  else
-  {
+  } else {
     fprintf (stderr, "no audio plugin available");
     goto away;
   }
@@ -1418,22 +1364,16 @@ int main (int argc, char **argv)
   config_value = g_key_file_get_string
     (config_file, "msid_config", "search_plugin", &error);
 
-  if (config_value)
-  {
+  if (config_value) {
     snprintf (buffer, 256, "%s/search/%s", PLUGIN_FILE_PATH, config_value);
     g_free (config_value);
 
-    if (g_file_test (buffer, G_FILE_TEST_EXISTS))
-    {
+    if (g_file_test (buffer, G_FILE_TEST_EXISTS)) {
       init_msid_plugin (&search_iface, buffer);
-    }
-    else
-    {
+    } else {
       goto away;
     }
-  }
-  else
-  {
+  } else {
     fprintf (stderr, "no search plugin available");
     goto away;
   }
@@ -1473,18 +1413,12 @@ int main (int argc, char **argv)
   }
 #endif
 
-  /*
-   * create containers
-   */
-
+  // create containers
   main_box = gtk_vbox_new (FALSE, 0);
 
   player_main_box = gtk_vbox_new (FALSE, 0);
 
-  /*
-   * create widgets
-   */
-  
+  // create widgets
   player_label = gtk_label_new ("Player");
   loader_label = gtk_label_new ("Search");
   image_label  = gtk_label_new ("Screenshot");
@@ -1492,16 +1426,10 @@ int main (int argc, char **argv)
   status_background = gtk_event_box_new();
   status_text       = gtk_label_new("");
 
-  /*
-   * pack widgets
-   */
-
+  // pack widgets
   gtk_container_add (GTK_CONTAINER(status_background), status_text);
 
-  /*
-   * pack containers
-   */
-
+  // pack containers
   loader_page = create_loader_page (msid_active_search_plugin, update_status_text);
 
   /*
@@ -1581,18 +1509,6 @@ int main (int argc, char **argv)
 
 #endif
 
-  /* BIG layout for desktop systems - make this look *good* */
-  /*
-#define UI_PAD 4
-
-
-
-  mother_container = gtk_hbox_new (FALSE, UI_PAD);
-  gtk_box_pack_start (GTK_BOX(mother_container), player_page, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX(mother_container), loader_page, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX(mother_container), image_page, FALSE, FALSE, 0);
-#endif
-  */
 
   main_menu = gtk_menu_item_new_with_label ("Menu");
 
