@@ -21,6 +21,7 @@
 
 GtkWidget *player_song_list;
 GtkWidget *player_play_button;
+GtkWidget *player_pause_button;
 GtkWidget *player_read_button;
 GtkWidget *player_del_button;
 GtkWidget *subsong_selector;
@@ -59,7 +60,7 @@ create_view_and_model (void)
 
   renderer = gtk_cell_renderer_text_new ();
 
-  //  g_object_set (G_OBJECT (renderer), "cell-background", "#888888", NULL);
+  // g_object_set (G_OBJECT (renderer), "cell-background", "#FF8800", NULL);
 
   /*
   GValue *value = (GValue *) malloc (sizeof(GValue));
@@ -365,20 +366,25 @@ create_player_page (void (*status_text_func) (const char*, const char*))
   ctrl_main_box = gtk_hbox_new (FALSE, 0);
   subsong_box = gtk_vbox_new (FALSE, 0);
 
-  player_play_button = create_ui_button ("PLAY");
+  player_play_button = create_ui_button ("STOP");
+  player_pause_button = create_ui_toggle_button ("PAUSE");
   player_read_button = create_ui_button ("REFRESH");
   player_del_button  = create_ui_button ("DEL");
 
+  gtk_widget_set_sensitive (player_play_button, FALSE);
+  gtk_widget_set_sensitive (player_pause_button, FALSE);
+
   subsong_selector = create_subsong_selector (1);
 
   player_song_list = create_view_and_model();
 
-  // TODO - fix bugs with pannable area
-  //scroll_win = hildon_pannable_area_new();
-
+#ifdef MAEMO5
+  scroll_win = hildon_pannable_area_new();
+#else
   scroll_win = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll_win),
                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+#endif
 
   /* --- signals --- */
 
@@ -388,6 +394,7 @@ create_player_page (void (*status_text_func) (const char*, const char*))
   /* --- packing --- */
 
   gtk_box_pack_start (GTK_BOX(ctrl_box), player_play_button, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX(ctrl_box), player_pause_button, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX(ctrl_box), player_read_button, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX(ctrl_box), player_del_button, TRUE, TRUE, 0);
 
